@@ -3,12 +3,12 @@ import logging
 import json
 import os
 import csv
-from py2neo.bulk import create_relationships, merge_relationships
 from typing import Set, List
 import gzip
 
 from graphio import defaults
 from graphio.helper import chunks, create_single_index, create_composite_index
+from graphio.queries import rels_create_unwind, rels_merge_unwind, rels_params_from_objects
 
 log = logging.getLogger(__name__)
 
@@ -92,21 +92,6 @@ class RelationshipSet:
     def __str__(self):
         return f"<RelationshipSet ({self.start_node_labels}; {self.start_node_properties})-[{self.rel_type}]->({self.end_node_labels}; {self.end_node_properties})>"
 
-    def __relationship_from_dictionary(self, start_node_properties, end_node_properties, properties):
-        """
-        Transform the input dictionary into the relationship data type used for py2neo.
-
-        :return: Relationship data.
-        """
-        start_node_data = tuple(start_node_properties[x] for x in self.fixed_order_start_node_properties)
-        if len(start_node_data) == 1:
-            start_node_data = start_node_data[0]
-        end_node_data = tuple(end_node_properties[x] for x in self.fixed_order_end_node_properties)
-        if len(end_node_data) == 1:
-            end_node_data = end_node_data[0]
-
-        return (start_node_data, properties, end_node_data)
-
     def add_relationship(self, start_node_properties: dict, end_node_properties: dict, properties: dict = None):
         """
         Add a relationship to this RelationshipSet.
@@ -126,10 +111,10 @@ class RelationshipSet:
                 list(start_node_properties.values()) + list(end_node_properties.values()) + list(rel_props.values()))
 
             if check_set not in self.unique_rels:
-                self.relationships.append(self.__relationship_from_dictionary(start_node_properties, end_node_properties, rel_props))
+                self.relationships.append((start_node_properties, end_node_properties, rel_props))
                 self.unique_rels.add(check_set)
         else:
-            self.relationships.append(self.__relationship_from_dictionary(start_node_properties, end_node_properties, rel_props))
+            self.relationships.append((start_node_properties, end_node_properties, rel_props))
 
     def all_property_keys(self) -> Set[str]:
         """
@@ -141,7 +126,7 @@ class RelationshipSet:
 
         # collect properties
         for r in self.relationships:
-            all_props.update(r[1].keys())
+            all_props.update(r[2].keys())
 
         return all_props
 
@@ -155,13 +140,10 @@ class RelationshipSet:
         :return:
         """
         start_node_property_types = {}
-        for i, p in enumerate(self.fixed_order_start_node_properties):
+        for p in self.start_node_properties:
             this_type = None
             for rel in self.relationships[:100]:
-                if isinstance(rel[0], tuple):
-                    value = rel[0][i]
-                else:
-                    value = rel[0]
+                value = rel[0][p]
                 type_of_value = type(value)
                 if not this_type:
                     this_type = type_of_value
@@ -172,14 +154,11 @@ class RelationshipSet:
             start_node_property_types[p] = this_type
 
         end_node_property_types = {}
-        for i, p in enumerate(self.fixed_order_end_node_properties):
+        for p in self.end_node_properties:
             this_type = None
             for rel in self.relationships[:100]:
 
-                if isinstance(rel[2], tuple):
-                    value = rel[2][i]
-                else:
-                    value = rel[2]
+                value = rel[1][p]
 
                 type_of_value = type(value)
                 if not this_type:
@@ -196,7 +175,7 @@ class RelationshipSet:
             this_type = None
             for rel in self.relationships[:100]:
                 try:
-                    value = rel[1][p]
+                    value = rel[2][p]
                     type_of_value = type(value)
                 except KeyError:
                     type_of_value = None
@@ -277,20 +256,13 @@ class RelationshipSet:
             for rel in self.relationships:
                 # create data for row
                 rel_csv_dict = {}
-                for i, k in enumerate(self.fixed_order_start_node_properties):
+                for k in self.start_node_properties:
+                    rel_csv_dict["a_{}".format(k)] = rel[0][k]
 
-                    if isinstance(rel[0], tuple):
-                        rel_csv_dict["a_{}".format(k)] = rel[0][i]
-                    else:
-                        rel_csv_dict["a_{}".format(k)] = rel[0]
+                for k in self.end_node_properties:
+                    rel_csv_dict["b_{}".format(k)] = rel[1][k]
 
-                for i, k in enumerate(self.fixed_order_end_node_properties):
-                    if isinstance(rel[2], tuple):
-                        rel_csv_dict["b_{}".format(k)] = rel[2][i]
-                    else:
-                        rel_csv_dict["b_{}".format(k)] = rel[2]
-
-                for k, v in rel[1].items():
+                for k, v in rel[2].items():
                     rel_csv_dict["rel_{}".format(k)] = v
 
                 writer.writerow(rel_csv_dict)
@@ -492,13 +464,10 @@ class RelationshipSet:
         log.debug('Batch Size: {}'.format(batch_size))
 
         # iterate over chunks of rels
+        q = rels_create_unwind(self.start_node_labels, self.end_node_labels, self.start_node_properties, self.end_node_properties, self.rel_type)
         for batch in chunks(self.relationships, size=batch_size):
-
-            create_relationships(graph.auto(),
-                                 batch,
-                                 self.rel_type,
-                                 start_node_key=(tuple(self.start_node_labels), *self.fixed_order_start_node_properties),
-                                 end_node_key=(tuple(self.end_node_labels), *self.fixed_order_end_node_properties))
+            query_parameters = rels_params_from_objects(batch)
+            graph.run(q, **query_parameters)
 
 
     def merge(self, graph, batch_size=None):
@@ -511,14 +480,11 @@ class RelationshipSet:
         log.debug('Batch Size: {}'.format(batch_size))
 
         # iterate over chunks of rels
+        q = rels_merge_unwind(self.start_node_labels, self.end_node_labels, self.start_node_properties,
+                               self.end_node_properties, self.rel_type)
         for batch in chunks(self.relationships, size=batch_size):
-
-            merge_relationships(graph.auto(),
-                                batch,
-                                self.rel_type,
-                                start_node_key=(tuple(self.start_node_labels), *self.fixed_order_start_node_properties),
-                                end_node_key=(tuple(self.end_node_labels), *self.fixed_order_end_node_properties)
-                                )
+            query_parameters = rels_params_from_objects(batch)
+            graph.run(q, **query_parameters)
 
 
     def create_index(self, graph):
@@ -592,13 +558,9 @@ def _yield_rels(csv_filepath, start_node_properties, end_node_properties, start_
 
     for row in rdr:
 
-        start_node_data = tuple(row[start_key_to_header[x]] for x in start_node_properties)
-        if len(start_node_data) == 1:
-            start_node_data = start_node_data[0]
+        start_node_data = dict(zip(start_node_properties, [row[start_key_to_header[x]] for x in start_node_properties]))
 
-        end_node_data = tuple(row[end_key_to_header[x]] for x in end_node_properties)
-        if len(end_node_data) == 1:
-            end_node_data = end_node_data[0]
+        end_node_data = dict(zip(end_node_properties, [row[end_key_to_header[x]] for x in end_node_properties]))
 
         # get properties
         properties = {}
@@ -606,8 +568,7 @@ def _yield_rels(csv_filepath, start_node_properties, end_node_properties, start_
             if k.startswith('rel_'):
                 k = k.replace('rel_', '')
                 properties[k] = v
-
-        yield (start_node_data, properties, end_node_data)
+        yield (start_node_data, end_node_data, properties)
 
     csvfile.close()
 
